;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc8ce7a123bc3298a7a9d5ac7157f791c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x30a068ad1106f0e85a26fa198dcf24ae . "./code.cg2js/cg-printing.fasl")
(#xb0f2e7fba6c8b300a54fc68032e572a4 . "./code.cg2js/cg-caret.fasl")
(#x62ff3a36027e84b31778c42aef963574 . "./code.cg2js/cg-item-list.fasl")
(#x4cef79f5d0d1ea1c0302258be4c7a0a7 . "./code.cg2js/cg-progress-indicator.fasl")
(#x953875c6e1b53af23225d60de5ee8128 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1ecbd3461301af3f3d4a279f039503b6 . "./code.cg2js/cg-outline.fasl")
(#x948dc81e99d6ab8b3fa28bd946d6ad1f . "./code.cg2js/cg-palette.fasl")
(#x1db2a3769ad73b0ae1f995323b4bdbb0 . "./code.cg2js/cg-selected-object.fasl")
(#x19c4c3fcc6addafc9a3921b1e5508a63 . "./code.cg2js/cg-status-bar.fasl")
(#xac52c707c161e058f0a4d438f3a6128b . "./code.cg2js/cg-wrap-string.fasl")
(#xbd35808c239113b02b31d99f3d1295bf . "./code.cg2js/cg-fill-texture.fasl")
(#xda968a952f42ba2656eea9e0362836ef . "./code.cg2js/cg-layout.fasl")
(#xf26a7135da0b2200f514ddda0c050eae . "./code.cg2js/cg-timer.fasl")
(#x86e3db85dbdd21448ce812ca842e9080 . "./code.cg2js/cg-dialog-item.fasl")
(#x383a5cb61d207199761ba6577f971a8a . "./code.cg2js/cg-button.fasl")
(#x25440f45dbf09d8e6f55e71ff2afa070 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7b8c7aacbab3e99d922b2bde209eaec1 . "./code.cg2js/cg-tray.fasl")
(#x5ee92f792d78bfc35a70d8a248894c58 . "./code.cg2js/cg-plot-widget.fasl")
(#xdc65a35e1aaf198f7c695e93a5fc7418 . "./code.cg2js/cg-pixmap.fasl")
(#xefe4177757619682171639ed51719b60 . "./code.cg2js/cg-font-dialog.fasl")
(#x5dc0d86ed8ad4177daa92ae613383ee5 . "./code.cg2js/cg-os-widget.fasl")
(#xec1658b73913b0395c89a780f7928db9 . "./code.cg2js/cg-text-or-combo.fasl")
(#x112819b270f6e4f608a70e166e488b20 . "./code.cg2js/cg-trackbar.fasl")
(#x1b3da64cd5f1be22ba309a5be93daffb . "./code.cg2js/cg-up-down-control.fasl")
(#x8b51890253f63443453909de8173d126 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbb62e40b06d66341665cbd7179da8c17 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xdf8a179c860e2150c5421728e000e8a6 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x7b77b0943a280ba3c8291cad5d43001c . "./code.cg2js/cg-bitmap-stream.fasl")
(#x6adb641ff888a3a36069df578666f071 . "./code.cg2js/cg-chart-widget.fasl")
(#xa7346e459051c83f1207182d0331f94b . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x08d85436119d8450e41d01ec130c7216 . "./code.cg2js/cg-group-box.fasl")
(#x1e3196d7f972815f3eab1eacfb34ffa0 . "./code.cg2js/cg-static-text.fasl")
(#x436f8596c0e78f456680169876146817 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8c3f8b2a3ba47a69a80721f26c3ac31c . "./code.cg2js/cg-color-dialog.fasl")
(#x967da96e41d6486b9b1b4697ac46e3e7 . "./code.cg2js/cg-intersect.fasl")
(#x1338809e3b83fc8daf226c8f8a79d82b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x78b598e74aa0f04085dd31d43096b333 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xbae783507b4bcb87bec9501ba6a0ff6b . "./code.cg2js/cg-html-dialog.fasl")
(#x113fc3677f285da2a72f46cd3533d754 . "./code.cg2js/cg-lettered-menu.fasl")
(#xc0c8d85656d56e5af264d9f6fd1064b2 . "./code.cg2js/cg-class-grid.fasl")
(#x9adb3518f18f2d4e7decb84c9b06c0f9 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x4a78eb2874c889e38107bad0ee101560 . "./code.cg2js/cg-calendar.fasl")
(#xcf3a6c0ca938458bd5be58ec2573fc8f . "./code.cg2js/cg-editable-text.fasl")
(#x50bbacc22b3aff2c1e5110fbf09736f3 . "./code.cg2js/cg-property.fasl")
(#xcb04dadb80d469fcc6bb6c87c60f3d4a . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xde0a8bfdfe5d6aeffe6f990c02d78fb1 . "./code.cg2js/cg-grid-widget.fasl")
(#xcbaf1a0c18696ffff792de71d902c9b0 . "./code.cg2js/cg-paren-matching.fasl")
(#x063bb5159dd88e0effac4a3b4f94448d . "./code.cg2js/cg-dev.fasl")
(#x1b231d3b7cdaac041c6bf13cab66a8d5 . "./code.cg2js/cg-drawable.fasl")
(#x0c2e056b3316175d7fa5c6520e51adb5 . "./code.cg2js/cg-curve.fasl")
(#x431fa6263b385fc00d6708a421ce0e02 . "./code.cg2js/cg-lisp-widget.fasl")
(#xc7e893046cd62e7f19eb9df183ea5030 . "./code.cg2js/cg-dde-utils.fasl")
(#xaacb5c33cc2bc7933f615506f0056431 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xdaf5592e84935c736d2d2db35705ebf0 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc17f6b42b16713c3ba2ddd8909ba0523 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xbb06ec21b2e036c932c061633eb49763 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5972417e765db59bf26b94296fb882e3 . "./code.cg2js/cg-lamp.fasl")
(#xd2921fbca3acf7300860b3e437d49323 . "./code.cg2js/cg-common-control.fasl")
(#x56ce1d828f5f0c78f0d5fb6db2168911 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xdb4fc9d49482abafe63008974d591f43 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd8a403e320e9ebd22ca2a5abc2aeb28f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x776fda057d03254e18bfafa7e5dffa98 . "./code.cg2js/cg-project-window.fasl")
(#xe053f9d8dd6875b5f6a9d75e7d912e07 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xf02accfbf13cb32d2a83c3db372f3137 . "./code.cg2js/cg-class-support.fasl")
(#x4ae905da78e889aa65bcb7775d77cfd8 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0e7f9515e51ffe4366b53e8d490c8834 . "./code.cg2js/cg-yes-no-list.fasl")
(#x3e8327b9fe7569bcd7f965419bf5323f . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x92ccd6e05b9f43e883f190d9a5d0b6d6 . "./code.cg2js/cg-string-dialog.fasl")
(#xe12cf71c8894204b85a5ee32d434ce77 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd4240439df9eccd4184f7d7ef0a034af . "./code.cg2js/cg-dropping-outline.fasl")
(#x4a8fdbf1bf7edcaa6a8480627edf0759 . "./code.cg2js/cg.fasl")
(#xace314e926637eed95dbb21867d7f144 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x014662f71122350c9258882f2b8f4eb0 . "./code.cg2js/cg-scaling-stream.fasl")
(#x4565b4f20abff03afcb90a3989018d7e . "./code.cg2js/cg-yes-no-string.fasl")
(#x663a4a5ba321b8ff1c09bb447977591d . "./code.cg2js/cg-icon-pixmap.fasl")
(#x8f087d585a02445345ad3a3299c89583 . "./code.cg2js/cg-web-browser.fasl")
(#x93bd18fc0e0e8e7363de78ed03abec57 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x017d7e2d32442783081251349229cd34 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x77f4dd233864d7536f05236f7a1dbc3b . "./code.cg2js/cg-message-dialog.fasl")
(#xd4043c744efb60701c75f0035585fc2e . "./code.cg2js/cg-comtab.fasl")
(#x2741a0c7c8a8370ac1de28126d4429d4 . "./code.cg2js/cg-base.fasl")
(#xe0db79a5d3363c10806eac404f6dc520 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x171198efabf24d3dbf6a64751edf303e . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd7f71f5d5e29c85848cf0c0947e06b27 . "./code.cg2js/cg-utility-dialog.fasl")
(#xacc5cd0e621fe9451b0f24167e8d01ce . "./code.cg2js/cg-object-editor.fasl")
(#x306c6ae786823b918d9cf32845d22559 . "./code.cg2js/cg-edit-in-place.fasl")
(#x099576d309cc1c7e4f2dbf49931dcbc7 . "./code.cg2js/cg-radio-button.fasl")
(#xae6313142594a62ae4426aa01108cd1f . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x8079547c376ede432d17c26c36948c65 . "./code.cg2js/cg-nodes.fasl")
(#x0064059f7facb94bede9c29ce096aa4d . "./code.cg2js/cg-get-position.fasl")
(#xb9a8d729b123e7267cc6725af06f6839 . "./code.cg2js/cg-directory-dialog.fasl")
(#x75015d3ba05ff81bd9a67685f10da62d . "./code.cg2js/cg-lisp-text.fasl")
(#xb8bdbe2fea4c91f52860680aa522c992 . "./code.cg2js/cg-combo-box.fasl")
(#xfd2db31e2045bd9b28514ccceed14a22 . "./code.cg2js/cg-text-widget.fasl")
(#x8d7181609d21a8a32d4854403d20aeb7 . "./code.cg2js/cg-list-view.fasl")
(#x1e3629e118dc8cce906520ab09e60944 . "./code.cg2js/cg-graphics-context.fasl")
(#xd6065a6a84e9530e4a1cf833f98510b2 . "./code.cg2js/cg-get-pixmap.fasl")
(#x6278ea27667d5d6631725ced00715315 . "./code.cg2js/cg-menu.fasl")
(#xe31af5c358edcbbca320f43fcb9853d3 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x074aebfb1bfac96f0d65b1d3d1cc9b74 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x47cc8189a5f1570eafe742a434767a80 . "./code.cg2js/cg-tooltip.fasl")
(#xf37201c65110a821cc228b2c0c5b47fd . "./code.cg2js/cg-html-widget.fasl")
(#x451a5da95de03e2173d42aba5587a5af . "./code.cg2js/cg-choice-list.fasl")
(#x1b318fc5f9debaba49c94fdde704e082 . "./code.cg2js/cg-icon.fasl")
(#x0843eb6b26ba26307b2960341250c358 . "./code.cg2js/cg-acache.fasl")
(#xd3fb94aa22a610467269c7a96319df4f . "./code.cg2js/cg-template-string.fasl")
(#x5f8884a156f30532afa7bd274617c52b . "./code.cg2js/cg-bitmap-pane.fasl")
(#x459c82076eb32b37eaa884182dd1b0b3 . "./code.cg2js/cg-toggling-widget.fasl")
(#x550fe437bc2dea8788cb1b3643687d73 . "./code.cg2js/cg-tab-control.fasl")
(#xcfa564afdab1448ac967d72549705efa . "./code.cg2js/cg-split-bar.fasl")
(#x2456e4c392091282ed037d11c731f240 . "./code.cg2js/cg-picture-widget.fasl")
(#x460873616dc6fc4885c50c83f7d9b9a9 . "./code.cg2js/cg-player.fasl")
(#xc41926eb5ad4da9e50d864aad6d2bfe2 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xeb249fe8b4235588cb492ad0b7f63fe6 . "./code.cg2js/cg-choose-printer.fasl")
(#xcea7361766ca0c9908df1cba34edfa43 . "./code.cg2js/cg-os-window.fasl")
(#x6573464b38a6d60291eb1f17eacb4f98 . "./code.cg2js/cg-scroll-bar.fasl")
(#x1f79c4e01ab1d7c36014ef0a9dc57d9e . "./code.cg2js/cg-clipboard.fasl")
(#x67766c41999bd090a5a72495accc16e8 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xd291a91d91b91ac79f78db7bf74c7bc6 . "./code.cg2js/cg-hotspot.fasl")
(#x2a33d6399bc9a43932278b53216e11a9 . "./code.cg2js/cg-check-box.fasl")
(#x5b53440873c9a60ea5662f895d39d9dd . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6f63761eb018b947c1fbde95b755773a . "./code.cg2js/cg-file-dialog.fasl")
(#x5d2e7523a3e4fa5df0766d2677c85852 . "./code.cg2js/cg-toolbar.fasl")
(#xf1e254794588501aa4c5299839f3f358 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x84cf24dd84801cd97c08f9eb16b916c8 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x6a552e86a2f409de38c3036a476a7fc7 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x9b7a1d9aa8c497af447fb7e8dbcf876a . "./code.idejs/ide-project.fasl")
(#x5edd29020f40f630c5094f8984574197 . "./code.idejs/ide-intro.fasl")
(#xa3de1f278bdf47684887e4d366c1d6b5 . "./code.idejs/ide-find-in-files.fasl")
(#x2b6a7ade8b584014e6a7f0b98f249f81 . "./code.idejs/ide-grapher.fasl")
(#x2174168df0577309857c976bfa3b0085 . "./code.idejs/ide-inspector.fasl")
(#x97c404769e7a6d04d96168aebb027658 . "./code.idejs/ide-editor.fasl")
(#xbe583ba5b3c47aac66f77afd7ccc0b1b . "./code.idejs/ide-window-list.fasl")
(#x24dcdf88ba7753b5a34566aaad88dad7 . "./code.idejs/ide-process-browser.fasl")
(#xeed2beaa9cb989055be73fc8df9579ec . "./code.idejs/ide-patch-dialog.fasl")
(#xa1a6704ef73da2191e82324c62c0ff3e . "./code.idejs/ide-class-grapher.fasl")
(#x2a6a59e339f803ec6b65bbac94256d69 . "./code.idejs/ide-base.fasl")
(#xc8122b16220728543f1478e399a5f848 . "./code.idejs/ide.fasl")
(#xea7e720bede7e67bcd29b6576b1119fd . "./code.idejs/ide-trace.fasl")
(#xb60e578950483a61c3eb69bb07735d7a . "./code.idejs/ide-form.fasl")
(#xfe255d8001fa961723c46fc1c9673f3c . "./code.idejs/ide-find-definitions.fasl")
(#xc81a5647bac570dcd3b812372d6f2289 . "./code.idejs/ide-apropos.fasl")
(#x930cd50f7b2798815abfd482e85b4fdb . "./code.idejs/ide-class-browser.fasl")
(#xfb9d363648dd3f30d92041ee43c90108 . "./code.idejs/ide-navigator.fasl")
(#x3c1075aacf631eb5524d7ea0d6d47354 . "./code.idejs/ide-clipboard.fasl")
(#x0d84edf8b7fd3a44f03d8a00133ab736 . "./code.idejs/ide-acache.fasl")
(#x91cf472630a67e7344dab96d18665609 . "./code.idejs/ide-keyshort.fasl")
(#xe642f2a432793289ac13529f2e381cfd . "./code.idejs/ide-package-list.fasl")
(#x493b9e70470f57933142f07bba4f916f . "./code.idejs/ide-dev.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
